/**
 *
 * Copyright (C) 2013, 2014 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

/**
 * \file main.c
 * \brief GA Triangulation Delaunay
 * \author Luis Felipe Restrepo Botero, Olivier Berthier
 * \date décembre 2013
 *
 */
#include "commun.h"
#include "vertex.h"
#include "simplex.h"
#include "delaunay.h"
#include "image.h"
#include "pixmap_io.h"




/*! variable externe permettant de lire les parametres sur le ligne de commande.*/
extern char *optarg;

/*! bascule pour autoriser ou interdire (0) les messages d'erreurs envoyes par getopt. */
extern int opterr;


/*! Tailles diverses du systemes de fenetrage. */
const double minX = 0,
  maxX = 1,
  minY = 0,
  maxY = 1,
  minZ = 0,
  maxZ = 1,
  margin = 10;


/*! Liste de vertex */
static vertex* ALL=NULL;

/*! File de priorité. */
static listSimplex* triangles=NULL;

/*! Identifiant de la fenêtre d'affichage. */
static int main_window;

/*! Mode d'affichage */
static int mode;

/*! Option de debugage */
static int debug;

/*! Affichage maillage / fil de fer. */
static int fill = 0;

// Parametre global de temps
//float t=0.0f;


/*! Angles de la caméra. */
static float alpha=-60.0f;
static float beta=0.0f;
static float delta=-138.0f;

/*! Etats de la souris. */
static int Mousekey=0,Mousex,Mousey;

/*! Affichage des cercles */
static int circle=0;

/*! Activation de la lumière OpenGL. */
static int light=1;

/*! Largeur de la fenêtre d'affichage. */
static int width=512;

/*! Hauteur de la fenêtre d'affichage. */
static int height=512;


/**
 * \fn int irand(int rangeLow, int rangeHigh)
 * \brief Fonction générant un nombre aléatoire entier entre deux nombres
 * \param rangeLow min
 * \param rangeHigh max
 * \return un nombre aléatoire
 */
int irand(int rangeLow, int rangeHigh) {
  return rangeLow+rand()%(rangeHigh-rangeLow);
}

/**
 * \fn double frand(double rangeLow, double rangeHigh) 
 * \brief Fonction générant un nombre aléatoire réel entre deux nombres.
 * \param rangeLow min
 * \param rangeHigh max
 * \return un nombre aléatoire
 */
double frand(double rangeLow, double rangeHigh) {
  return rangeLow+rand()*((rangeHigh-rangeLow)/(double)RAND_MAX);
}

/**
 * \fn double gaussrand()
 * \brief Fonction générant un nombre aléatoire selon une loi gaussienne
 * \return un nombre aléatoire
 */
double gaussrand(){
  static double U, V;
  static int phase = 0;
  double Z;

  if(phase == 0) {
    U = (rand() + 1.) / (RAND_MAX + 2.);
    V = rand() / (RAND_MAX + 1.);
    Z = sqrt(-2 * log(U)) * sin(2 * M_PI * V);
  } else
    Z = sqrt(-2 * log(U)) * cos(2 * M_PI * V);

  phase = 1 - phase;
       
  return Z;
}

/**
 * \fn double gaussian(double min, double max)
 * \brief Fonction permettant d'obtenir un nombre aléatoire généré selon une loi gaussienne recadrée entre deux nombres.
 * \param min min
 * \param max max
 * \return un nombre aléatoire
 */
double gaussian(double min, double max){
  double X = gaussrand();
  /* 99% X is in [-2.57, 2.57] */
  X = ((double)(min+max))/2 + X/2.57 * (((double)(max-min))/2);
  if (X < min)
    return min;
  else if (X > max)
    return max;
	
  return X;
}

/**
 * \fn void print_commandes()
 * \brief Procédure d'affichage de l'aide des commandes du programme.
 */
void print_commandes(){
  printf("Commandes:\n");
  printf(" -Les touches 'ESC' et 'S' permettent de quitter le programme.\n");
  printf(" -La touche 'H' permet d'afficher l'aide des commandes.\n");
  printf(" -La touche 'C' permet de lancer un contrôle de la triangulation. Cette commande peut prendre un certain temps.\n");
  printf(" -En mode 2D la touche 'D' permet de tracer les cercles circonscrit de chaque triangles.\n");
  printf(" -En mode 3D la touche 'L' permet d'activer/désactiver la lumière.\n");
  printf(" -En mode 3D la touche 'A' permet d'activer/désactiver le remplissage des triangles.\n");
  printf(" -En mode 3D la rotation est effectuée avec les flèches directionnelles.\n");
  printf(" -En mode debug la touche 'ESPACE' permet d'effectuer une itération.\n");
  printf("\n");
}

/**
 * \fn void print_help(char* argv[])
 * \brief Procédure d'affichage de l'aide du programme.
 * \param argv[] liste des options de lancement du programme.
 */
void print_help(char* argv[]){
  printf("Usage: %s [OPTIONS]\n", argv[0]) ;
  printf("Options :\n") ;
  printf(" -d              \
Activer le mode de debugage. Ce mode permet de suivre la triangulation itérations par itérations. Pour effectuer une itération, il faut appuyer sur la touche ESPACE.\n") ;
  printf(" -f N            \
Indique le nombre de facettes N maximum à calculer.\n") ;
  printf(" -h              \
Afficher l'aide.\n") ;
  printf(" -g              \
Initialiser les points selon une loi de probabilité gaussienne.\n") ;
  printf(" -i N            \
Indique le nombre d'itérations N maximum que l'on souhaite effectuer lors de la triangulation.\n") ;
  printf(" -l <filename>   \
Fichier image à charger pour obtenir l'amplitude de chaque point généré. Il faut que l'image soit au format PGM binaire (P5).\n") ; 
  printf(" -m [2|3]        \
Indiquer le mode 2D ou 3D. Par défaut le mode activé est le mode %dD.\n",DEF_MODE);
  printf(" -n N            \
Indiquer le nombre de vertices N à générer. Par défaut le nombre de vertices générés est %d.\n",DEF_NB_VERTICES) ;
  printf(" -o              \
Activer l'oprtimisation, c'est à dire que la triangulation d'une face s'arrête lorsque l'écart est de zero même s'il reste des vertices dans le simplex.\n");
  printf(" -s N            \
Indiquer l'écart maximal de tout point à son projeté verticale sur le plan du triangle qui le contient. Cette valeur doit être exprimée en pourcentage par rapport à la hauteur unitaire utilisée pour la génération de l'amplitude des vertices.\n");
  printf(" -z N            \
Définir la hauteur minimal, c'est à dire la hauteur du carré initial. Cette valeur doit être exprimée en pourcentage par rapport à la hauteur unitaire utilisée pour la génération de l'amplitude des vertices. Ne fonctionne pas avec l'option -l.\n");
  printf(" -Z N            \
Définir la hauteur maximal. Cette valeur doit être exprimée en pourcentage par rapport à la hauteur unitaire utilisée pour la génération de l'amplitude des vertices. Ne fonctionne pas avec l'option -l.\n");
  printf("\n");
  print_commandes();
}


/**
 * \fn vertex* initMap(double zMin)
 * \brief Fonction de génération des 4 vertex initiaux.
 * \param zMin Hauteur à laquelle sont générés les vertex.
 * \return liste de vertex
 */
vertex* initMap(double zMin){
  //les premiers points
  vertex* first=newVertex();  
  first->coords[0]=minX;
  first->coords[1]=minY;
  first->coords[2]=zMin;//frand(minZ, maxZ);
 
  vertex* current = first;
  {
    vertex* v =newVertex();
    v->coords[0]=maxX;
    v->coords[1]=minY;
    v->coords[2]=zMin;//frand(minZ, maxZ);
    current->link->next = v;
    v->link->prev = current;  
    current = v;    
  }

  {
    vertex* v =newVertex();
    v->coords[0]=maxX;
    v->coords[1]=maxY;
    v->coords[2]=zMin;//frand(minZ, maxZ);
    current->link->next = v;
    v->link->prev = current;  
    current = v;
  }
 
  {
    vertex* v =newVertex();
    v->coords[0]=minX;
    v->coords[1]=maxY;
    v->coords[2]=zMin;//frand(minZ, maxZ);
    current->link->next = v;  
    v->link->prev = current;  
    current = v;
  }
  return first;
}


/**
 * \fn vertex* initFromImage(char* filename,int nbPoints, int gauss)
 * \brief Fonction de génération de N vertex depuis un fichier image PGM binaire (P5).
 * \param filename Nom du fichier
 * \param nbPoints Nombre de vertex
 * \param gauss Utilisation d'une génération gaussienne en X et Y
 * \return une liste de vertex
 */
vertex* initFromImage(char* filename,int nbPoints, int gauss){
  printf("Lecture du fichier %s\n",filename);  
  /*
    FILE *f ;
    struct image* map;
    f=fopen(filename,"r");
    map = lecture_image(f);
    fclose(f) ;
    //liberation_image(map);
    */

  unsigned char *data;
  int width, height;
  if((data = load_pixmap(filename, &width, &height)) == NULL)
    return NULL;//TODO : erreur

  vertex* first = initMap(0);
  vertex* current = first;
  while(current->link->next!=NULL)
    current=current->link->next;
  int i=0,j=0;
  while(nbPoints){
    vertex* v =newVertex();
    
    if(gauss){
      v->coords[0]=gaussian(minX+EPSILON,maxX-EPSILON);
      v->coords[1]=gaussian(minY+EPSILON,maxY-EPSILON);     
      // v->coords[0]=grand(minX+EPSILON,maxX-EPSILON,1,0.3);
      // v->coords[1]=grand(minY+EPSILON,maxY-EPSILON,1,0.3);     
    }else{
      v->coords[0]=frand(minX+EPSILON,maxX-EPSILON);
      v->coords[1]=frand(minY+EPSILON,maxY-EPSILON); 
    }
    i=(v->coords[0]*width);
    j=(v->coords[1]*height);
    v->coords[2]=(*(data+i*width+j));
    v->coords[2]/=255;
    current->link->next = v;
    v->link->prev = current;
    current = v;  
    nbPoints--;
  }
  return first;
}


/**
 * \fn vertex* initRandomPoints(int nbPoints, double zMin, double zMax, int gauss)
 * \brief Fonction de génération de N vertex aléatoires.
 * \param nbPoints Nombre de vertex
 * \param zMin Hauteur minimal
 * \param zMax Hauteur maximal
 * \param gauss Utilisation d'une génération gaussienne en X et Y
 * \return une liste de vertex
 */
vertex* initRandomPoints(int nbPoints, double zMin, double zMax, int gauss){
  vertex* first = initMap(zMin);
  vertex* current = first;
  while(current->link->next!=NULL)
    current=current->link->next;


  while(nbPoints){
    
    vertex* v =newVertex();
    if(gauss){
      v->coords[0]=gaussian(minX+EPSILON,maxX-EPSILON);
      v->coords[1]=gaussian(minY+EPSILON,maxY-EPSILON);     
      // v->coords[0]=grand(minX+EPSILON,maxX-EPSILON,1,0.3);
      // v->coords[1]=grand(minY+EPSILON,maxY-EPSILON,1,0.3);     
    }else{
      v->coords[0]=frand(minX+EPSILON,maxX-EPSILON);
      v->coords[1]=frand(minY+EPSILON,maxY-EPSILON); 
    }
    v->coords[2]=frand(zMin+EPSILON,zMax-EPSILON);
    current->link->next = v;
    v->link->prev = current;
    current = v;  
    nbPoints--;
      
  }
  return first;
}

/**
 * \fn void display3D(void)
 * \brief Procédure d'affichage 3D
 */
void display3D(void){
  vertex* n = newVertex();
  vertex* a = newVertex();
  vertex* b = newVertex();

  glColor3d(0.3, 0.3, 0.3);
  glBegin(GL_TRIANGLES) ;    

#if HEAP>0
  int i;
  if(triangles!=NULL){
    for(i = 0; i<triangles->size;i++){
      simplex* s=triangles->ss[i];
      if (s!=NULL){  

	diff(s->sommets[1],s->sommets[0],a);
	diff(s->sommets[2],s->sommets[0],b);
	crossProduct(a,b,n);
	normalize(n);
	glNormal3f(n->coords[0],n->coords[1],n->coords[2]);

	glVertex3f(s->sommets[0]->coords[0]-0.5, s->sommets[0]->coords[1]-0.5, s->sommets[0]->coords[2]-0.5);    
	glVertex3f(s->sommets[1]->coords[0]-0.5, s->sommets[1]->coords[1]-0.5, s->sommets[1]->coords[2]-0.5);    
	glVertex3f(s->sommets[2]->coords[0]-0.5, s->sommets[2]->coords[1]-0.5, s->sommets[2]->coords[2]-0.5);    

      }
    }
  }
#else
  if(triangles!=NULL){
    simplex* s=triangles->s;
    while (s!=NULL){  

      diff(s->sommets[1],s->sommets[0],a);
      diff(s->sommets[2],s->sommets[0],b);
      crossProduct(a,b,n);
      normalize(n);
      glNormal3f(n->coords[0],n->coords[1],n->coords[2]);

      glVertex3f(s->sommets[0]->coords[0]-0.5, s->sommets[0]->coords[1]-0.5, s->sommets[0]->coords[2]-0.5);    
      glVertex3f(s->sommets[1]->coords[0]-0.5, s->sommets[1]->coords[1]-0.5, s->sommets[1]->coords[2]-0.5);    
      glVertex3f(s->sommets[2]->coords[0]-0.5, s->sommets[2]->coords[1]-0.5, s->sommets[2]->coords[2]-0.5);    

      s=s->prev;
    }
  }
#endif
  glEnd();

  deleteVertex(a);
  deleteVertex(b);
  deleteVertex(n);


  //glFlush();
}

/**
 * \fn void display2D(void)
 * \brief Procédure d'affichage 2D
 */
void display2D(void){

  double r,x,y;
  double t;
  int i,j;
#if HEAP>0
  if(triangles!=NULL){
    for(i = 0; i<triangles->size;i++){
      simplex* s=triangles->ss[i];
      if (s!=NULL){  
	glColor3f(0.9, 0.9, 0.9);
	glBegin(GL_LINE_LOOP);
	glVertex2f(s->sommets[0]->coords[0],s->sommets[0]->coords[1]);	   
	glVertex2f(s->sommets[1]->coords[0],s->sommets[1]->coords[1]);	   
	glVertex2f(s->sommets[2]->coords[0],s->sommets[2]->coords[1]);	   
	circumcircle(s,&r,&x,&y);     
	glEnd();
	//
	//     
	if(circle){
	  glColor3f(1, 0, 0);
	  glBegin(GL_LINE_LOOP);
	  t=0;
	  for(j = 0; j < CIRCLE_VERTICES; ++j){
	    glVertex2f(x + r *  cos(t), y + r * sin(t));
	    t += 2 * M_PI /  CIRCLE_VERTICES;
	  }
	  glEnd();
	}
      }
    }
  }
#else
  if(triangles!=NULL){
    simplex* s=triangles->s;
    while (s!=NULL){
      glColor3f(0.9, 0.9, 0.9);
      glBegin(GL_LINE_LOOP);
      glVertex2f(s->sommets[0]->coords[0],s->sommets[0]->coords[1]);	   
      glVertex2f(s->sommets[1]->coords[0],s->sommets[1]->coords[1]);	   
      glVertex2f(s->sommets[2]->coords[0],s->sommets[2]->coords[1]);	   
      circumcircle(s,&r,&x,&y);
      s=s->prev;
      glEnd();
      //
      //     
      if(circle){
	glColor3f(1, 0, 0);
	glBegin(GL_LINE_LOOP);
	t=0;
	for(i = 0; i < CIRCLE_VERTICES; ++i){
	  glVertex2f(x + r *  cos(t), y + r * sin(t));
	  t += 2 * M_PI /  CIRCLE_VERTICES;
	}
	glEnd();
      }
    }
  }
#endif
  glFlush();
}

/**
 * \fn void GlutRendering()
 * \brief Procédure d'affichage global
 */
void GlutRendering(){

  if(mode==3){
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();


    glTranslatef(0.0f,0.0f,-3.0f);	

    glRotatef(alpha,1.0f,0.0f,0.0f); 
    glRotatef(beta,0.0f,1.0f,0.0f);
    glRotatef(delta,0.0f,0.0f,1.0f);


    if(fill)
      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL); 
    else
      glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

    if(light){
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0);
    }else{
      glDisable(GL_LIGHTING);
      glDisable(GL_LIGHT0);
    }
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);

    glBegin(GL_LINES);
    glColor3f(1.0, 0.0, 0.0);
    glVertex3f(-0.5,-0.5,-0.5); 
    glVertex3f(0.5,-0.5,-0.5); 
    glColor3f(0.0, 1.0, 0.0);
    glVertex3f(-0.5,-0.5,-0.5); 
    glVertex3f(-0.5,0.5,-0.5); 
    glColor3f(0.0, 0.0, 1.0);
    glVertex3f(-0.5,-0.5,-0.5); 
    glVertex3f(-0.5,-0.5,0.5); 
    glEnd();

    glPointSize(1.0f);
    display3D();

    glutSwapBuffers();

  }else{

    glColor3f(0.0, 0.0, 0.0);
    glClear(GL_COLOR_BUFFER_BIT);
    glShadeModel(GL_FLAT);
    glColor3f(1.0, 1.0, 1.0);
    display2D();

  }
}


/**
 * \fn void MouseMove(int x, int y)
 * \brief mouse
 * \param x
 * \param y
 */
void MouseMove(int x, int y){
  Mousex = x;
  Mousey = y;
}

/**
 * \fn void MousePush(int button, int state, int x, int y)
 * \brief mouse
 * \param button
 * \param state
 * \param x
 * \param y
 */
void MousePush(int button, int state, int x, int y){
  Mousex = x;
  Mousey = y;
  Mousekey=button;
}

// Rafraichissement
/* void GlutIdle(void){ */
/*   // Augmentation du parametre de temps */
/*   t+=1.0; */
/*   GlutRendering(); */
/* } */

/**
 * \fn void processNormalKeys(unsigned char key, int x, int y)
 * \brief Quitter avec la touche Echap ou S
 */
void processNormalKeys(unsigned char key, int x, int y) {

  if (key == 27 || key == 115 ||  key == 83) {
    glutDestroyWindow(main_window);
#if HEAP>0
    //TODO nettoyage
    //    deleteListSimplex(triangles);
#else
    deleteListSimplex(triangles);
#endif
    exit(0);
  }else if(key == 99 || key == 67){
    printf("Contrôle de la triangulation en cours...\n");
    if(!checkDelaunayFull(triangles,1,0)) 
      printf("DELAUNAY : Erreur InCircle\n");
    else
      printf("DELAUNAY : OK\n");
  }else if(key == 100 || key == 68){
    circle =! circle;
    glutPostRedisplay();
  }else if(key == 104 || key == 72){
    print_commandes();
  }else if(key == 108 || key == 76){
    light =! light;
    glutPostRedisplay();
  }else if(key == 97 || key == 65){
    fill =!fill;
    glutPostRedisplay();
  }else if(key == 32){
    if(debug){
      delaunay_it(triangles);
      glutPostRedisplay();
    }
  }

}


/**
 * \fn void SpecialKeyboard(int key, int x, int y)
 * \brief special keys
 */
void SpecialKeyboard(int key, int x, int y){

  switch (key) {
  case GLUT_KEY_LEFT :
    delta+=0.95*2;
    break;
  case GLUT_KEY_RIGHT :
    delta-=0.95*2;
    break;
  case GLUT_KEY_UP :
    alpha+=0.95*2;
    break;
  case GLUT_KEY_DOWN :
    alpha-=0.95*2;
    break;
  }

  GlutRendering();
}

/**
 * \fn void winInit (void)
 * \brief ...
 */
void winInit (void){

  if(mode==3){
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_ALPHA | GLUT_DEPTH);
  }else{
    glutInitDisplayMode(GLUT_SINGLE | GLUT_RGBA);  
  }


  glutInitWindowSize(width, height);  
  // Fenetre initiale

  glutInitWindowPosition(100, 100);  
  main_window = glutCreateWindow("Triangulation Delaunay");  


  glutDisplayFunc(GlutRendering);
  //glutFullScreen();

  // Rafraichissement
  //  glutIdleFunc(&GlutIdle);

  // Changement de taille
  //  glutReshapeFunc(Resize);

  // Clavier
  glutKeyboardFunc(processNormalKeys);


  // Souris
  //  glutMouseFunc(MousePush);
  //glutMotionFunc(MouseMove);
  
  // Initialise les parametres de rendu
  glClearColor (0.0f, 0.0f, 0.0f, 1.0);

  if(mode==3){
    glutSpecialFunc(SpecialKeyboard);

    glClearDepth(1.0);
    glDepthFunc(GL_LEQUAL);
    glEnable(GL_DEPTH_TEST);	
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    glShadeModel(GL_SMOOTH);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();				// Reset The Projection Matrix

    gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);
  }else{
    gluOrtho2D(minX, maxX, minY, maxY);
  }
}



/**
 * \fn int main(int argc, char **argv)  
 * \brief Entrée du programme
 *
 * \return EXIT_SUCCESS - Arrêt normal du programme.
 */
int main(int argc, char **argv)  {
  srand(time(0));  
  int option,nbPoints=DEF_NB_VERTICES,optimise=0;
  unsigned long nbIterations = 0;
  unsigned long nbFaces=0;
  double seuil=0;
  double zMin=minZ;
  double zMax=maxZ;

  char *load_filename = NULL ;

  mode = DEF_MODE;
  debug = 0;
  int gauss = 0;
  opterr = 0; /* set off std error messages in case wrong option is chosen */
  while ((option = getopt(argc, argv, "df:gi:l:m:n:s:z:Z:o")) != EOF){
    switch (option){
    case 'd':
      debug = 1;
      break;
    case 'f': 
      if ((sscanf(optarg, "%ld", &nbFaces) != 1) || nbFaces < 0)
	nbFaces = 0;
      break;
    case 'g':
      gauss = 1;
      break;
    case 'i': 
      if ((sscanf(optarg, "%ld", &nbIterations) != 1) || nbIterations < 0)
	nbIterations = 0;
      break;
    case 'l':
      load_filename = optarg ;
      break ;
    case 'm': 
      if ((sscanf(optarg, "%d", &mode) != 1) || (mode != 2  && mode != 3))
	mode = DEF_MODE;
      break;
    case 'n': 
      if ((sscanf(optarg, "%d", &nbPoints) != 1) || nbPoints <= 0)
	nbPoints = DEF_NB_VERTICES;
      break;
    case 'o':
      optimise = 1;
      break;
    case 's': 
      if ((sscanf(optarg, "%lf", &seuil) != 1) || seuil < 0)
	seuil = 0;
      break;
    case 'z': 
      if ((sscanf(optarg, "%lf", &zMin) != 1) || zMin < 0)
	zMin = minZ;
      else
	zMin = minZ + (zMin / 100.0) * maxZ;
      break;
    case 'Z': 
      if ((sscanf(optarg, "%lf", &zMax) != 1) || zMax < 0)
	zMax = maxZ;
      else
	zMax = minZ + (zMax / 100.0) * maxZ;
      break;
    case '?': 
    case 'h': 
    default : 
      print_help(argv);
      return EXIT_SUCCESS;  
      break ;	
    }
  }

  
  assert(nbPoints>0);
  assert(nbIterations>=0);
  assert(mode==2 || mode==3);
  assert(zMin<zMax);
  // le seuil est saisie en % => passage à l'échelle [0-1]
  seuil  /= 100;

  glutInit(&argc, argv);  
  winInit(); 

  
  //Initialiser les points aléatoires
  printf("Generation de %d vertices\n",nbPoints);     

  if(load_filename != NULL)
    ALL = initFromImage(load_filename,nbPoints,gauss); 
  else
    ALL = initRandomPoints(nbPoints, zMin, zMax,gauss); 
  
  clock_t tStart = clock() ;
  if(debug){
    triangles=newListSimplex(2*(sizeList(ALL)-4)+2);// liste des simplexes validés
    splitQuad(ALL,triangles); 
    delaunay_it(triangles);
  }else
    triangles= delaunay(ALL,nbFaces,nbIterations,seuil,optimise);

  printf("Nombre de simplex générés : %d\n",triangles->size);
  printf("CPU TIME USED : %.4fs\n",
	 (double)(clock() - tStart)/CLOCKS_PER_SEC) ;

  printf("DRAW\n");
  glutMainLoop();  

  glutDestroyWindow(main_window);
  return EXIT_SUCCESS;  
}  
