/**
 *
 * Copyright (C) 2013, 2014 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
/**
 * \file vertex.h
 * \brief GA Triangulation Delaunay
 * \author Luis Felipe Restrepo Botero, Olivier Berthier
 * \date décembre 2013
 *
 */
#ifndef VERTEX_H
#define VERTEX_H

/**
 * \file vertex.h
 * \brief Vertex
 *
 * Structutres et fonctions pour vertex.
 *
 */

#include "commun.h"

#include "simplex.h"


/**
 * \struct p_vertex
 * \brief Structure pour enregistrer les liens d'un vertex. 
 */
typedef struct 
{
  /*! Vertex suivant */
  struct _vertex * next; 
  /*! Vertex précédent */
  struct _vertex * prev; 
} p_vertex; 

/** 
 * \struct vertex
 * \brief Structure pour representer un point. 
 */
typedef struct _vertex
{
  /*! Coordonnées du vertex */
  double coords[DIM];
  /*! Structure de liens du vertex */ 
  p_vertex * link;
  /*! Compteur de références */
  unsigned int compteur;
  struct vertex *(*newVertex)();
  void*(*deleteVertex)();
} vertex; 

/**
 * \fn vertex *newVertex()
 * \return Vertex initialisé
 * \brief Fonction d'initalisation d'un nouveau vertex.
 */
vertex *newVertex();

/**
 * \fn void deleteVertex()
 * \return Vertex initialisé
 * \brief Fonction de nettoyage d'un vertex.
 */
void deleteVertex(vertex* v);
 /**
 * \fn void printList(vertex* T)
 * \param T liste de vertex
 * \brief Procédure permettant d'afficher une liste de vertex.
 */
void printList(vertex* T);

/**
 * \fn void sizeList(vertex* T)
 * \param T liste de vertex
 * \return taille de la liste
 * \brief Procédure permettant d'afficher la taille d'une liste de vertex.
 */
int sizeList(vertex* T);

/**
 * \fn void printVertex(vertex* v)
 * \param v Vertex à afficher
 * \brief Procédure d'affichage des information d'un vertex.
 */
void printVertex(vertex* v);


/** 
 * \fn  InCircle (vertex *A, vertex *B, vertex *C, vertex *Z)
 * \param A the first vertex determining the circle
 * \param B the second vertex determining the circle
 * \param C the last vertex determining the circle
 * \param Z the vertex to be tested against the circle \f$\Gamma(A,B,C)\f$.
 * \brief determines wether vertex \a Z lies ouside, on, or inside
 * the circle passing through \a A, \a B and \a C.
 *
 * Let \f$\Gamma(ABC)\f$ be the circle around vertices \f$A, B, C\f$.
 * Guibas and Stolfi have shown that the relative position of \f$Z\f$
 * and \f$\Gamma(A,B,C)\f$ is equivalent to computing the sign 
 * of determinant:
 * \f[
 \left |
 \begin{array}{cccc}
 x_A     & y_A     & x_A^2+y_A^2  & 1 \\
 x_B     & y_B     & x_B^2+y_B^2  & 1 \\
 x_C     & y_C     & x_C^2+y_C^2  & 1 \\
 x_Z     & y_Z     & x_Z^2+y_Z^2  & 1 \\
 \end{array}
 \right |
 * \f]
 * corresponding to the relative position of vertex \f$Z\f$ and
 * the plane through the projections of vertices \f$A,B,C\f$
 * on the paraboloid of revolution with equation
 * \f[ z = x^2+y^2. \f]
 * According to Guibas & Stolfi's result, the current routine should return:
 *  -  -1 (OUT) if vertex \f$Z\f$ is outside circle \f$\Gamma(A,B,C)\f$,
 *  -  0 (ON)  if vertex \f$Z\f$ lies on circle \f$\Gamma(A,B,C)\f$, and
 *  -  1 (IN)  if vertex \f$Z\f$ lies inside circle \f$\Gamma(A,B,C)\f$.   
 *
 *  For convenience, as the only disturbing case is 
 *  "\f$D\f$ lies inside \f$\Gamma(A,B,C)\f$",
 *  the routine actually returns: 
 *     1 (IN)  if vertex \f$Z\f$ strictly lies inside circle \f$\Gamma(A,B,C)\f$,
 *     0       otherwise.
 *
 *  \warning Vertices \f$A,B,C\f$ are ASSUMED neither to be aligned or equal.
 *           Overflow might arise otherwise. User must check this condition
 *           with function Angle() before calling present function.
 */
int InCircle (vertex *A, vertex *B, vertex *C, vertex *Z);


/**
 * \fn int puissance(vertex *a, vertex *b, vertex *c)
 * \brief Fonction permettant si un point c se trouve à droite, à gauche ou sur la droite (ab).
 *
 * \param a Adresse du vertex a 
 * \param b Adresse du vertex b 
 * \param c Adresse du vertex c 
 * \return La position du point c par rapport à la droite (ab)
 */
int puissance(vertex *, vertex *, vertex *);

/**
 * \fn int inTriangle(vertex *a, vertex *b, vertex *c, vertex* p)
 * \brief Fonction qui retourne DEDANS si le point p est à l'interieur du triangle abc, DEHORS à l'exterieur et SUR sur la frontière
 *
 * \param a Adresse du vertex a 
 * \param b Adresse du vertex b 
 * \param c Adresse du vertex c 
 * \param p Adresse du vertex p 
 * \return La position du point p par rapport à abc
 */
int inTriangle(vertex *a, vertex *b, vertex *c, vertex* p);

/**
 * \fn int checkPolygoneConvexe(struct _vertex** v, int direction)
 * \brief Fonction qui test si un polygone de taille fix (POLYGONE_SIZE) est convexe.
 * \param v Tableau de vertices formant un polygone
 * \param direction Direction à contrôler (AGAUCHE ou ADROITE)
 * \return 1 si le polygone est convexe, sinon 0
 */
int checkPolygoneConvexe(struct _vertex** v, int direction);

/**
 * \fn int checkPolygoneDirection(struct _vertex** v)
 * \brief Fonction qui retourne le sens de rotation des vertex d'un polygone.
 * \param v Tableau de vertices formant un polygone
 * \return ADROITE ou AGAUCHE
 */
int checkPolygoneDirection(struct _vertex** v);


/**
 * \fn int leftPolygone(struct _vertex** v)
 * \brief Fonction qui retourne l'indice du point le plus à gauche d'un polygone.
 * \param v Tableau de vertices formant un polygone
 * \return indice du point le plus à gauche
 */
int leftPolygone(struct _vertex** v);

/**
 * \fn void crossProduct(const vertex* u, const vertex* v, vertex* r)
 * \brief Procédure qui calcule le produit croisé entre deux vertex A^B=C
 * \param u Vertex A
 * \param v Vertex B
 * \param r Vertex C 
 */
void crossProduct(const vertex* u, const vertex* v, vertex* r);

/**
 * \fn void diff(const vertex* u, const vertex* v, vertex* r)
 * \brief Procédure qui calcule la différence entre deux vertex A-B=C
 * \param u Vertex A
 * \param v Vertex B
 * \param r Vertex C 
 */
void diff(const vertex* u, const vertex* v, vertex* r);

/**
 * \fn void normalize(vertex* v)
 * \brief Procédure permettant de normaliser un vertex.
 * \param v Vertex
 */
void normalize(vertex* v);

/**
 * \fn void addRef(vertex* v, int s)
 * \brief Procédure permettant d'incrémenter le compteur de références à un vertex.
 * \param v Vertex
 * \param s Nombre de références
 */
void addRef(vertex* v, int s);

/**
 * \fn void removeRef(vertex* v, int s)
 * \brief Procédure permettant décrémenter le compteur de références à un vertex.
 * \param v Vertex
 * \param s Nombre de références
 */
void removeRef(vertex* v, int s);

#endif //VERTEX_H
